Search.setIndex({docnames:["index","intro"],envversion:50,filenames:["index.rst","intro.rst"],objects:{},objnames:{},objtypes:{},terms:{"case":1,"class":1,"default":1,"final":1,"import":1,"public":1,And:1,For:1,One:1,The:[0,1],There:1,These:1,UIs:1,Use:1,_name_chang:1,about:1,abov:1,access:1,act:1,action:1,add:1,aim:1,all:1,allow:[0,1],alreadi:1,also:1,although:1,alwai:1,angularj:1,ani:1,api:1,app:1,applic:[0,1],approach:1,async:1,attribut:1,automat:1,avail:1,avoid:1,awesom:1,backend:1,base:[0,1],beauti:1,better:1,between:1,bind:1,bit:1,body_html:1,bridg:1,broken:1,browser:[0,1],build:1,call:1,can:[0,1],chang:1,changelog:0,cleanup:1,code:1,command:1,commun:1,complet:1,connect:1,contain:1,context:1,correspond:1,coverag:1,creat:[0,1],css:[0,1],current:1,data:1,declar:1,def:1,depend:1,desktop:1,develop:1,directli:1,directori:1,document:1,doe:1,doesn:1,don:1,done:1,easi:1,easier:1,easili:1,either:0,ensur:1,entir:[0,1],ex22_vuejs_demo:1,exampl:0,exec_:1,file:1,find:1,first:1,fix:1,follow:1,fred:1,from:1,full:1,fulli:1,further:1,get:0,greet:1,guess:1,guess_typ:1,gui:1,guru:1,hand:1,hastrait:1,have:1,hello:1,henc:1,here:1,how:1,html:[0,1],htmlwidget:1,http:1,index:0,input:1,instal:0,instanc:1,interact:1,interfac:[0,1],introduct:0,ioloop:1,issu:1,itself:1,javascript:[0,1],jigna:1,know:1,let:1,librari:1,like:1,limit:1,linux:1,listen:1,live:1,load:1,localhost:1,look:1,made:1,mai:1,make:1,mani:1,meant:1,mime:1,mock:1,model:1,modul:0,much:1,multipl:1,name:1,nativ:1,necessari:1,nice:1,nose:1,nosetest:1,notic:1,now:1,number:1,off:1,one:1,onli:1,oppos:1,option:1,over:1,page:[0,1],peopl:1,person:1,pip:1,place:1,pleas:1,power:1,previou:1,primari:1,problem:1,procedur:1,process:1,produc:1,provid:1,pull:1,pure:1,purpos:1,pypi:1,pyqt:1,pysid:1,python3:1,python:[0,1],qapplic:1,qt4:1,qtgui:1,read:1,readm:1,reason:1,refer:1,referenc:1,releas:1,reload:1,render:1,requir:1,respond:1,rich:1,root:1,run:1,sai:1,sampl:1,seamless:1,search:0,see:1,selenium:1,self:1,separ:1,sequenc:1,serv:1,server:1,set:1,setup:1,sever:1,show:1,similar:1,simpl:1,sinc:1,solut:1,sourc:1,space:1,specif:1,start:0,str:1,style:1,support:1,sure:1,sync:1,take:1,task:1,team:1,templat:1,test:0,than:1,thei:1,them:1,thi:1,though:1,thousand:1,thread:1,todai:1,togeth:1,toolkit:1,tornado:1,trait:1,tutori:1,two:1,txt:1,type:1,updat:1,use:1,user:[0,1],using:1,varieti:1,veri:1,versa:1,version:1,via:1,vice:1,view:[0,1],visual:1,vue:1,wai:1,web:[0,1],webapp:1,when:1,which:1,who:1,widget:1,window:1,wire:1,work:1,worldwid:1,worri:1,write:1,written:1,you:[0,1],your:1,yourself:1},titles:["Welcome to the Jigna documentation!","Introduction"],titleterms:{changelog:1,document:0,exampl:1,get:1,indic:0,instal:1,introduct:1,jigna:0,start:1,tabl:0,test:1,welcom:0}})